import 'dart:async';

import 'package:dart_dokr/src/core/constants.dart';
import 'package:dart_dokr/src/core/file_system.dart';
import 'package:dart_dokr/src/core/process.dart';
import 'package:dart_dokr/src/dokr_command.dart';
import 'package:dart_dokr/src/models/config.dart';
import 'package:mason_logger/mason_logger.dart';

class DoctorCommand extends DokrCommand {
  @override
  String get name => 'doctor';

  @override
  String get description => 'Check your system for potential problems.';

  @override
  FutureOr<int>? run() {
    logger.info('Running doctor...');

    final ok = Process.checkMultipleInstalled(
      [
        'docker',
        'docker-compose',
      ],
    );

    for (final item in ok) {
      if (item.$2) {
        logger.success('${item.$1} is available.');
      } else {
        logger.err('${item.$1} is not available.');
        exitCode = ExitCode.unavailable.code;
      }
    }

    // Check for `dokr.yaml` config file. (or file specified by `--config` flag)

    final config = globalResults?['config'] as String?;
    if (config != null) {
      logger.info('Using config file: $config');
    } else {
      logger.info('Using default config file: $kDefaultConfigFileName');
    }

    // Try to parse config file.
    final configExists =
        FileSystem.checkExists(config ?? kDefaultConfigFileName);

    if (configExists) {
      logger.success('Config file found.');
    } else {
      logger.err('Config file not found.');
      exitCode = ExitCode.unavailable.code;
    }

    Config configParsed;

    try {
      configParsed = FileSystem.parseConfig(config ?? kDefaultConfigFileName);
      logger
        ..success('Config parsed successfully.')
        ..detail('Config data dir: ${configParsed.dataDir}')
        ..detail('Config config dir: ${configParsed.configDir}')
        ..detail('Config stacks dir: ${configParsed.stacksDir}')
        ..detail('Config env files: ${configParsed.envFiles}')
        ..detail(
          'Config name template: ${configParsed.name.template.template}',
        );
    } catch (e) {
      logger
        ..err('Config file could not be parsed.')
        ..detail(e.toString());
      exitCode = ExitCode.unavailable.code;

      return super.run();
    }

    // Check if config directories exist.
    final dataDirExists = FileSystem.checkUriDirExists(configParsed.dataDir);
    final configDirExists =
        FileSystem.checkUriDirExists(configParsed.configDir);
    final stacksDirExists =
        FileSystem.checkUriDirExists(configParsed.stacksDir);

    if (dataDirExists) {
      logger.success('Data directory exists.');
    } else {
      logger.err('Data directory does not exist.');
      exitCode = ExitCode.unavailable.code;
    }

    if (configDirExists) {
      logger.success('Config directory exists.');
    } else {
      logger.err('Config directory does not exist.');
      exitCode = ExitCode.unavailable.code;
    }

    if (stacksDirExists) {
      logger.success('Stacks directory exists.');
    } else {
      logger.err('Stacks directory does not exist.');
      exitCode = ExitCode.unavailable.code;
    }

    // Check if config env files exist.
    final List<Uri> missingEnvFile = [];
    for (final envFile in configParsed.envFiles) {
      final envFileExists = FileSystem.checkUriExists(envFile);
      if (!envFileExists) {
        missingEnvFile.add(envFile);
        break;
      }
    }
    if (missingEnvFile.isEmpty) {
      logger.success('All env files exist.');
    } else {
      logger
        ..err('Not all env files exist.')
        ..detail('Missing env files: $missingEnvFile');
      exitCode = ExitCode.unavailable.code;
    }

    return super.run();
  }
}
