import 'dart:async';

import 'package:dart_dokr/src/core/docker.dart';
import 'package:dart_dokr/src/core/stacks.dart';
import 'package:dart_dokr/src/dokr_command.dart';
import 'package:mason_logger/mason_logger.dart';

class StartCommand extends DokrCommand {
  StartCommand() {
    argParser.addFlag(
      'force',
      abbr: 'f',
      help: 'Force restart a stack even if it is already running.',
    );
  }

  @override
  String get name => 'start';

  @override
  String get description => 'Start or restart one or more stacks';

  @override
  FutureOr<int>? run() async {
    // Get stack names from arguments.
    final stackArgs = argResults!.rest;
    final stacks = Stacks.listStacks(config);
    final force = argResults!['force'] as bool;

    Progress? progress;

    for (final stack in stackArgs) {
      if (!stacks.contains(stack)) {
        logger.err('Stack not found: $stack');
        continue;
      }

      // Check if stack is already running.
      final docker = Docker(
        stackFile: Stacks.findStackFile(config, stack)!,
        stackName: stack,
        config: config,
      );

      try {
        final isRunning = await docker.isRunning();
        logger.detail('Stack $stack is running: $isRunning');

        if (!isRunning) {
          progress = logger.progress('Starting stack $stack');
          await docker.start();
          progress.complete('Stack $stack started');
        }

        if (isRunning) {
          if (force) {
            progress = logger.progress('Restarting stack $stack');
            await docker.restart();
            progress.complete('Stack $stack restarted');
          } else {
            logger
              ..warn('Stack $stack is already running')
              ..info('Use --force to restart');
            exitCode = ExitCode.cantCreate.code;
          }
        }
      } catch (e) {
        if (progress != null) {
          progress.fail(e.toString());
        } else {
          logger.err(e.toString());
        }
        continue;
      }
    }

    return super.run() ?? ExitCode.success.code;
  }
}
