/// A template for generating names.
/// Usage:
/// ```dart
/// final template = NameTemplate.fromString('{ .Stack }.compose.yaml');
/// final values = <String, String>{
///   'stack': 'my_stack',
/// };
/// final rendered = template.render(values);
/// print(rendered); // my_stack.compose.yaml
/// ```
class NameTemplate {
  /// Creates a new [NameTemplate].
  const NameTemplate({
    required this.template,
    required this.parts,
  });

  /// Creates a new [NameTemplate] from a [template] string.
  factory NameTemplate.fromString(String template) {
    final parts = <TemplatePart>[];
    final buffer = StringBuffer();
    var isEscaped = false;
    var isVariable = false;

    for (var i = 0; i < template.length; i++) {
      final char = template[i];

      if (char == r'\') {
        isEscaped = true;
        continue;
      }

      if (char == '{' && !isEscaped) {
        isVariable = true;
        continue;
      }

      if (char == '}' && !isEscaped) {
        isVariable = false;
        var value = buffer.toString().trim().toLowerCase();
        // Remove the leading dot.
        if (value.startsWith('.')) {
          value = value.substring(1);
        }
        parts.add(
          TemplatePart(
            isVariable: true,
            value: value,
          ),
        );
        buffer.clear();
        continue;
      }

      if (isVariable) {
        buffer.write(char);
        continue;
      }

      parts.add(
        TemplatePart(
          isVariable: false,
          value: char,
        ),
      );
    }

    return NameTemplate(
      template: template,
      parts: parts,
    );
  }

  /// The template string.
  final String template;

  /// The template string parsed into a list of [TemplatePart]s.
  final List<TemplatePart> parts;

  /// Returns the [template] string with all variables replaced with [values].
  String render(Map<String, String> values) {
    final buffer = StringBuffer();

    for (final part in parts) {
      if (part.isVariable) {
        buffer.write(values[part.value]);
      } else {
        buffer.write(part.value);
      }
    }

    return buffer.toString();
  }

  @override
  String toString() => 'NameTemplate(template: $template, parts: $parts)';
}

/// A part of a [NameTemplate].
class TemplatePart {
  /// Creates a new [TemplatePart].
  const TemplatePart({
    required this.isVariable,
    required this.value,
  });

  /// Whether this part is a variable.
  final bool isVariable;

  /// The value of this part.
  final String value;

  @override
  String toString() => 'TemplatePart(isVariable: $isVariable, value: $value)';
}

void main(List<String> args) {
  final template = NameTemplate.fromString('{ .Stack }.compose.yaml');
  final values = <String, String>{
    'stack': 'my_stack',
  };
  final rendered = template.render(values);
  print(rendered);
}
