import 'dart:convert';
import 'dart:io' as io;

import 'package:dart_dokr/src/core/logging.dart';
import 'package:mason_logger/mason_logger.dart';

abstract class Process {
  static Logger get logger => Logging.instance;

  static io.ProcessResult runSync(
    String executable,
    List<String> arguments, {
    String? workingDirectory,
    Map<String, String>? environment,
    bool includeParentEnvironment = true,
    bool runInShell = false,
    Encoding? stdoutEncoding = io.systemEncoding,
    Encoding? stderrEncoding = io.systemEncoding,
  }) {
    if (workingDirectory != null) {
      logger.detail('> cd $workingDirectory');
    }
    logger.detail('> $executable ${arguments.join(' ')}');

    return io.Process.runSync(
      executable,
      arguments,
      workingDirectory: workingDirectory,
      environment: environment,
      includeParentEnvironment: includeParentEnvironment,
      runInShell: runInShell,
      stdoutEncoding: stdoutEncoding,
      stderrEncoding: stderrEncoding,
    );
  }

  static Future<io.ProcessResult> run(
    String executable,
    List<String> arguments, {
    String? workingDirectory,
    Map<String, String>? environment,
    bool includeParentEnvironment = true,
    bool runInShell = false,
    Encoding? stdoutEncoding = io.systemEncoding,
    Encoding? stderrEncoding = io.systemEncoding,
    bool withProgress = false,
    String? progressMessage,
    String? successMessage,
    String? errorMessage,
    bool logOutput = true,
  }) async {
    Progress? progress;

    if (withProgress && progressMessage != null) {
      progress = logger.progress(progressMessage);
    }

    if (workingDirectory != null) {
      logger.detail('> cd $workingDirectory');
    }
    logger.detail('> $executable ${arguments.join(' ')}');

    final result = await io.Process.run(
      executable,
      arguments,
      workingDirectory: workingDirectory,
      environment: environment,
      includeParentEnvironment: includeParentEnvironment,
      runInShell: runInShell,
      stdoutEncoding: stdoutEncoding,
      stderrEncoding: stderrEncoding,
    );

    if (progress != null) {
      if (result.exitCode == 0 && successMessage != null) {
        progress.complete(successMessage);
      } else if (result.exitCode != 0 && errorMessage != null) {
        progress.fail(errorMessage);
      } else {
        progress.cancel();
      }
    }

    if (logOutput) {
      if (result.stdout != null &&
          (result.stdout! as String).trim().isNotEmpty) {
        logger.detail(result.stdout!.toString());
      }
      if (result.stderr != null &&
          (result.stderr! as String).trim().isNotEmpty) {
        logger.err(result.stderr!.toString());
      }
    }

    return result;
  }

  static bool checkInstalled(String executable, {bool throwOnError = false}) {
    try {
      final result = runSync('which', [executable]);
      if (result.exitCode != 0) {
        logger.detail('$executable not found!');
        if (throwOnError) {
          throw Exception(
            '$executable not found!',
          );
        }
        return false;
      }
      logger.detail('$executable found');
      return true;
    } on io.ProcessException {
      logger.detail('$executable not found!');
      if (throwOnError) {
        throw Exception(
          '$executable not found!',
        );
      }
      return false;
    }
  }

  static List<(String, bool)> checkMultipleInstalled(
    List<String> executables, {
    bool throwOnError = false,
  }) {
    final results = <(String, bool)>[];
    for (final executable in executables) {
      results.add(
        (executable, checkInstalled(executable, throwOnError: throwOnError)),
      );
    }
    return results;
  }
}
