import 'dart:io';

import 'package:args/args.dart';
import 'package:args/command_runner.dart';
import 'package:cli_completion/cli_completion.dart';
import 'package:dart_dokr/src/commands/create_command.dart';
import 'package:dart_dokr/src/commands/doctor_command.dart';
import 'package:dart_dokr/src/commands/list_command.dart';
import 'package:dart_dokr/src/commands/start_command.dart';
import 'package:dart_dokr/src/commands/stop_command.dart';
import 'package:dart_dokr/src/core/constants.dart';
import 'package:dart_dokr/src/core/logging.dart';
import 'package:dart_dokr/src/core/version.dart';
import 'package:mason_logger/mason_logger.dart';

class DokrCommandRunner extends CompletionCommandRunner<int> {
  DokrCommandRunner() : super(kExecutableName, 'Manage Docker Stacks.') {
    argParser.addFlags();
    addCommand(DoctorCommand());
    addCommand(ListCommand());
    addCommand(CreateCommand());
    addCommand(StartCommand());
    addCommand(StopCommand());
  }

  /// The [Logger] instance.
  final Logger _logger = Logging.instance;

  @override
  Future<int> run(Iterable<String> args) async {
    try {
      final results = parse(args);
      Logging.instance = _logger.copyWith(
        level: results['verbose'] == true ? Level.verbose : Level.info,
      );
      return await runCommand(results) ?? ExitCode.success.code;
    } on FormatException catch (e) {
      _logger
        ..err(e.message)
        ..info('')
        ..info(usage);
      return ExitCode.usage.code;
    } on UsageException catch (e) {
      _logger
        ..err(e.message)
        ..info('')
        ..info(e.usage);
      return ExitCode.usage.code;
    } on ProcessException catch (error) {
      _logger.err(error.message);
      return ExitCode.unavailable.code;
    } catch (error) {
      _logger.err('$error');
      return ExitCode.software.code;
    }
  }

  @override
  Future<int?> runCommand(ArgResults topLevelResults) async {
    if (topLevelResults.command?.name == 'completion') {
      await super.runCommand(topLevelResults);
      return ExitCode.success.code;
    }

    int? exitCode = ExitCode.unavailable.code;
    if (topLevelResults['version'] == true) {
      _logger.info(packageVersion);
      exitCode = ExitCode.success.code;
    } else {
      exitCode = await super.runCommand(topLevelResults);
    }
    return exitCode;
  }
}

extension on ArgParser {
  void addFlags() {
    addFlag(
      'version',
      negatable: false,
      help: 'Print the current version.',
    );
    addFlag(
      'verbose',
      abbr: 'v',
      negatable: false,
      help: 'Print verbose output.',
    );
    addOption(
      'config',
      abbr: 'c',
      help: 'Specify an alternate config file.',
      defaultsTo: kDefaultConfigFileName,
      valueHelp: 'path',
    );
  }
}

extension on Logger {
  Logger copyWith({
    LogTheme? theme,
    Level? level,
  }) =>
      Logger(
        theme: theme ?? this.theme,
        level: level ?? this.level,
      );
}
